## --------------------------------------------------------------
## Name: functional-tradeoffs.R
## Description: Code to calculate functional redundancy and functional dispersion for all terrestrial ecoregions
## Author: Robert S.C. Cooke, R.S.Cooke@soton.ac.uk
## Date: November 2017 
## Inputs: eco (dataframe of Species x Site data), td_mi (dataframe of complete imputed trait data)
## Outputs: fr_mi (dataframe of functional metrics)
## --------------------------------------------------------------

#### --------------------------------------------------------------
#### Step 0: setup ####
#### --------------------------------------------------------------

### Set up directory and packages ###

# set up directory if needed (I use a project with the sub-folder data for the inputs/outputs)

if(!require("pacman")) install.packages("pacman")
pacman::p_load(dplyr, tibble, fossil, FD, multirich)

# dplyr: data manipulation # calls: select, %>%, mutate_at, mutate, rename, distinct, inner_join, left_join 
# tibble: used to convert column to rownames # calls: column_to_rownames, rownames_to_column
# fossil: used to create composition matrix # calls: create.matrix
# FD: calculates functional dispersion and gower dissimilarity # calls: gowdis, fdisp
# multirich: used to calculate unique trait combinations # calls: mvfd

#### --------------------------------------------------------------
#### Step 1: load ####
#### --------------------------------------------------------------

# load ecoregion data
eco <- read.csv("species_site.csv")

# load trait data
td_mi <- read.csv("trait_mi_selected_data.csv") %>% 
  select(-class) %>% 
  # drop class information
  mutate_at(vars(body_mass_median, litter_clutch_size), log10) %>% 
  # log10-transform specified traits (body mass, litter size, generation length)
  mutate(hab_breadth = sqrt(hab_breadth))
  # square-root transform habitat breadth

#### Check loaded data ####

# species not found in any ecoregion
no_eco <- setdiff(td_mi$binomial, eco$binomial)
# Acrocephalus rehsei - endemic to Nauru
# Pteropus howensis - endemic to Ontong Java Atoll in the Solomon Islands - islands are too small to be included in neighbouring ecoregion AA0119
# Melomys rubicola - now extinct

# remove missing ecoregion species from trait data
td_mi <- td_mi %>% 
  filter(!binomial %in% no_eco)

# test if the site and trait data sets contain the same number of species? 
isTRUE(nrow(td_mi) == nrow(distinct(eco, binomial))) 
# should be TRUE

#### --------------------------------------------------------------
#### Step 2: create ####
#### --------------------------------------------------------------

#### Set up site data (eco_mi_m) ####

## eco_mi_m ##

# create speciesxsite matrix
eco_mi_m <- t(create.matrix(as.data.frame(eco), tax.name = "binomial", locality = "eco")) 
# eco has to be a definite data.frame

## eco_mi_m_pool ##

# add species pool
eco_mi_m_pool <- rbind(matrix(1, nrow = 1, ncol = ncol(eco_mi_m), dimnames = list(c("pool"), c(colnames(eco_mi_m)))), eco_mi_m)
# add pool as a site with every species present

## e ##

# number of ecoregions assessed
e_mi <- dim(eco_mi_m)[1]
e_mi_pool <- e_mi+1

#### Functional dispersion ####

# function to z-transform data
scale_z <- function(x){
  (x - mean(x, na.rm=TRUE))/sd(x, na.rm=TRUE)
}

## td_mi_z ##

# z-transform trait data
td_mi_z <- td_mi %>%
  mutate_at(vars(-binomial), scale_z) %>% 
  # scale (zero mean and unit variance) all traits
  as.data.frame %>% 
  # convert to true dataframe ato prevent 'Warning message: Setting row names on a tibble is deprecated.'
  column_to_rownames(var = "binomial")
  # add species names to rownames (needed for gd function)

## ts_mi ##

# combine td_mi and eco_rm data
ts_mi <- list(td_mi_z, eco_mi_m_pool); names(ts_mi) <- c("td_mi","eco_mi")

## gd_rm ##

# calculate gower species x species distance matrix based on traits
gd_mi <- gowdis(ts_mi$td_mi)

## tb_stur_mi ##

# function to bin data using sturges algorithm
cut_bin_stur <- function(x) {
  h <- hist(x, breaks = "Sturges")
  as.numeric(cut(x, breaks = h$breaks))
}

# bin data using sturges algorithm as the number of bins
tb_stur_mi <- td_mi %>% 
  mutate_at(vars(body_mass_median, litter_clutch_size, hab_breadth), funs(cut_bin_stur)) %>% 
  # bin data
  as.data.frame %>% 
  # convert to true dataframe to prevent 'Warning message: Setting row names on a tibble is deprecated.'
  column_to_rownames(var = "binomial") 
  # convert species names to rownames

#### --------------------------------------------------------------
#### Null Model ####
#### --------------------------------------------------------------

# IF RUNNING NULL MODEL
# randomizes species composition (given the observed level of species richness)

## glob_spp ##

# create global species pool
glob_spp <- eco %>% 
  # dataframe of species composition per ecoregion for all ecoregions and IUCN mammals 1235 (data.frame)
  distinct(binomial)
# list of unique species names across globe (global pool)

## eco_spp ##

# species richness per ecoregion
eco_spp <- table(eco$eco) %>% 
  # species richness per ecoregion
  as.data.frame %>% 
  # convert table to dataframe
  dplyr::rename(eco = Var1, spp = Freq)
  # rename columns

# function to sample from species pool (global pool) equal to number of species in ecoregion
eco_sample <- lapply(1:nrow(eco_spp), function(x) {
  sample_n(glob_spp, eco_spp[x, "spp"]) %>% 
    mutate(eco = eco_spp[x, "eco"])
})

es <- bind_rows(eco_sample) %>% 
  mutate(eco = as.character(eco))

# then use es instead of eco for Step 3
# run in bash script as a job array on a high-performance computer 
# email author R.S.Cooke@soton.ac.uk for bash script and/or R script to bring null runs together

#### --------------------------------------------------------------
#### Step 3: functional metrics ####
#### --------------------------------------------------------------

#### Functional dispersion ####

## fd_mi ##

# calculate functional dispersion
fd_mi <- fdisp(gd_mi, ts_mi$eco_mi)

# modify functional dispersion output
fd_mi <- as.data.frame(fd_mi$FDis) %>%
  # extract data and convert to dataframe
  rownames_to_column("eco") %>% 
  # convert ecoregions to explicit column
  dplyr::rename(fdisp = `fd_mi$FDis`)
  # rename FDis column

# save: fd_mi
saveRDS(fd_mi, "data/df_fd_mi.rds")
# dataframe of functional dispersion (data.frame)

# load: fd_mi
#fd_mi <- readRDS("data/df_fd_mi.rds")
# dataframe of functional dispersion results (data.frame)

#### Load in functions ####

# fun_fg_spp - Function to generate species attributions to functional units
# fun_met - Function to calculate functional metrics: UTC, functional redundancy

# Offload these into separate scripts within an R project to keep it tidy

## --------------------------------------------------------------
## Name: fun-fg-spp-function.R
## Description: Function to generate species attributions to functional units
## Author: R.S.C. Cooke, R.S.Cooke@soton.ac.uk
## Date: October 2016
## Outputs: Function named 'fun_fg_spp'
## Args:
## tdf = Dataframe of trait data in bins
## em = Species x Site matrix
## e = Number of ecoregions assessed
## name = Name of the method to attach to objects being saved
## --------------------------------------------------------------

fun_fg_spp <- function(tdf, em, e, name) {
  
  ## gp ##
  
  # dataframe of all unique trait combinations for the trait data
  gp <- tdf %>% 
    distinct %>%
    # unique trait combinations
    mutate(group = 1:nrow(.))
    # add an identifying column called group to the dataframe
  
  ## fg_spp ##
  
  tdf <- tdf %>%  
    rownames_to_column(var = "binomial")
    # convert rownames to explicit column for use in join
  
  # dataframe of species attributions to functional units
  fg_spp <- inner_join(tdf, gp)
  
  # dataframe of number of species per functional unit
  count <- table(fg_spp$group) %>% 
    # count species per functional unit
    as.data.frame(., stringsAsFactors = FALSE) %>% 
    # convert to dataframe
    mutate(Var1 = as.numeric(Var1))
    # convert to numeric to allow join
  
  # join number of species per functional unit to species attributions
  fg_spp <- fg_spp %>% 
    left_join(count, by = c("group" = "Var1")) %>% 
    # join count to dataframe
    dplyr::select(binomial, group, count = Freq, everything())
    # reorder columns
  
  # save: fg_spp
  saveRDS(fg_spp, paste0("data/df_fg_spp_", name, ".rds"))
  # species attributions to functional groups
  
  return(fg_spp)
  
}

## --------------------------------------------------------------
## Name: fun-met-function.R
## Description: Function to calculate functional metrics: UTC, functional redundancy
## Author: R.S.C. Cooke, R.S.Cooke@soton.ac.uk
## Date: October 2017
## Outputs: Function named 'fun_met'
## Args:
## tdf = Dataframe of trait data in bins
## em = Matrix of Species x Site data
## fd = Dataframe of functional dispersion results
## e = Number of ecoregions assessed
## name = Name of the method to attach to objects being saved
## --------------------------------------------------------------

fun_met <- function(tdf, em, fd, e, name) {
  
  #### Generate species attributions to functional units ####
  
  ## fg_spp ##
  
  # run fun_fg_spp function
  fg_spp <- fun_fg_spp(tdf = tdf, em = em, e = e, name = name)
  
  #### Species richness per ecoregion ####
  
  eco_spp <- table(eco$eco) %>% 
    # species richness per ecoregion
    as.data.frame %>% 
    # convert table to dataframe
    dplyr::rename(eco = Var1, spp = Freq) %>% 
    # rename columns
    mutate(eco = as.character(eco))
  
  #### Unique Trait Combinations ####
  
  ## utc_pool ##
  
  # calculate utc for the species pool and use this for scaling purposes
  utc_pool <- mvfd(as.matrix(tdf), em, 0, calc.ovr = 0) 
  # 0 is for resolution
  # doesn't run with overlap on
  pool_traitspace <- utc_pool$utc[1]
  # full trait space
  
  ## utc_eco ##
  
  # calculate sutc (scaled utc) for every ecoregion
  utc_eco <- mvfd(as.matrix(tdf), em, 0, traitspace = pool_traitspace, calc.ovr = 0) 
  
  ## utc_df ##
  
  # create dataframe of ecoregions, sutc, utc and spp
  utc_df <- as.data.frame(cbind(rownames(em), utc_eco$utc), stringsAsFactors = FALSE) %>% 
    # extract data from mvfd function into dataframe
    dplyr::rename(eco = V1, utc = V2) %>% 
    # rename columns of dataframe
    mutate_each(vars(utc), as.numeric)
    # convert variables to numeric
  
  ## fr ##
  
  fr <- left_join(eco_spp, utc_df, by = "eco") %>%
    # join species richness to UTC data
    mutate(fred = spp/utc) %>%
    # calculate functional redundancy %>% 
    left_join(fd, by = "eco")
    # join functional dispersion data
  
  # save: fr
  #saveRDS(fr, paste0("data/df_fr_", name, ".rds"))
  # dataframe of functional metrics standardised between ecoregions (data.frame)
  
  return(fr)
  
}

## fr_mi ##

# run fr_utc function
fr_mi <- fun_met(tdf = tb_mi, em = eco_mi_m_pool, fd = fd_mi, e = e_mi_pool, name = "mi")

# load: fr_mi (diet5)
fr_mi <- readRDS("data/df_fr_mi.rds")
# dataframe of functional metrics for all ecoregions (data.frame)
